/*
*	refal.h - описание типов данных представляемой реализации языка Рефал.
*	© 2005, А.А.Владимиров.
*
*	Данная программа является свободным программным обеспечением.
*	Вы вправе распространять ее и/или модифицировать в соответствии
*	с условиями версии 2, либо, по вашему выбору, с условиями
*	более поздней версии Стандартной Общественной Лицензии GNU,
*	опубликованной Free Software Foundation.
*
*	Мы распространяем данную программу в надежде на то, что она будет вам
*	полезной, однако НЕ ПРЕДОСТАВЛЯЕМ НА НЕЕ НИКАКИХ ГАРАНТИЙ, в том числе
*	ГАРАНТИИ ТОВАРНОГО СОСТОЯНИЯ ПРИ ПРОДАЖЕ и ПРИГОДНОСТИ ДЛЯ ИСПОЛЬЗОВАНИЯ
*	В КОНКРЕТНЫХ ЦЕЛЯХ. Для получения более подробной информации ознакомьтесь
*	со Стандартной Общественной Лицензией GNU.
*
*	Вместе с данной программой вы должны были получить экземпляр
*	Стандартной Общественной Лицензии GNU. Если вы его не получили,
*	сообщите об этом в
*	
*	Free Software Foundation, Inc.,
*	59 Temple Place, Suite 330,
*	Boston, MA 02111-1307
*	USA
*
*/

# include <stdio.h>
# include <stdlib.h>

/*
*	Структура LITERO представляет собой основной в рефал-системе тип данных.
*	Объекты этого типа используются в трёх основных качествах:
*		I) В качестве элемента поля зрения и аналогичных списков.
*		II) В качестве описания переменной.
*		III) В качестве описания формулы подстановки.
*	
*	I. При использовании литеры в качестве элемента поля зрения
*	значения поля "->tip" могут быть следующими:
*
*	0) Макроцифра. Значением поля "->dat" является целое число "->dat.cif"
*	типа (unsigned int), величина которого заключена между 0 и 4294967295.
*
*	1) Левая литероидная скобка. Значением поля "->dat" является
*	указатель "->dat.ref" типа (LITERO *) на парную правую скобку.
*
*	2) Правая литероидная скобка. Значением поля "->dat" является
*	указатель "->dat.ref" типа (LITERO *) на парную левую скобку.
*
*	# Литероидные скобки используются для выделения части выражения,
*	# образующей неделимый символ (натуральное число). Между парными
*	# литероидными скобками должна быть заключена непустая цепочка макроцифр,
*	# представляющих собой разряды 4294967296-ичного представления числа,
*	# следующие в порядке от младших к старшим. При этом появление нуля
*	# в качестве старшего разряда допускается лишь в том случае,
*	# когда этот разряд является единственным (и число в целом равно нулю).
*	# 
*	# ПРИМЕР: натуральное число 1000000000000 представляется в виде
*	#
*	# 			[ 3567587328 232 ]
*	#
*	# Здесь через "[" и "]" обозначены, соответственно, левая и правая
*	# литероидные скобки; числа между ними обозначают макроцифры.
*
*	3) Левая структурная скобка. Значением поля "->dat" является
*	указатель "->dat.ref" типа (LITERO *) на парную правую скобку.
*
*	4) Правая структурная скобка. Значением поля "->dat" является
*	указатель "->dat.ref" типа (LITERO *) на парную левую скобку.
*
*	5) Левая функциональная скобка. Значением поля "->dat" является
*	указатель "->dat.funk" типа (FUNKCIO *) на подлежащую применению функцию.
*	
*	6) Правая функциональная скобка. Значением поля "->dat" является
*	указатель "->dat.ref" типа (LITERO *) на парную левую функциональную
*	скобку.
*	
*	7) Переменная. Значением параметра "->dat" является указатель "->dat.ref"
*	типа (LITERO *) на описание переменной.
*
*	Значением поля "->ant" является указатель на предыдущую литеру из списка.
*	Значением поля "->sekv" является указатель на следующую литеру из списка.
*
*	II. При использовании литеры в качестве описания переменной, значения
*	поля "->tip" могут быть следующими:
*	
*	0) 'Активная' s-переменная. Внутри образца данный тип отвечает
*	самому левому вхождению переменной с данным именем. Внутри правой части
*	формулы подстановки данный тип отвечает вхождению переменной,
*	значение которого будет строиться копированием (а не переносом)
*	некоторого участка исходного выражения. Значение поля "->dat" имеет смысл
*	только для переменной внутри правой части формулы подстановки:
*	в этом случае значение поля "->dat.ref" представляет собой указатель
*	на описание переменной из образца, значение которой подлежит копированию.
*
*	1) 'Пассивная' s-переменная. Внутри образца данный тип отвечает
*	вхождению переменной, левее которой уже имелись вхождения переменной
*	с тем же именем. Внутри правой части формулы подстановки данный тип
*	отвечает вхождению переменной, значение которого будет строиться переносом
*	(а не копированием) некоторого участка исходного выражения. Значение
*	поля "->dat.ref" представляет собой указатель на описание самого левого
*	вхождения переменной с данным именем в образец (для переменной из образца),
*	или указатель на описание переменной из образца, значение которой
*	подлежит переносу.
*	
*	2) 'Активная' t-переменная. Внутри образца данный тип отвечает
*	самому левому вхождению переменной с данным именем. Внутри правой части
*	формулы подстановки данный тип отвечает вхождению переменной,
*	значение которого будет строиться копированием (а не переносом)
*	некоторого участка исходного выражения. Значение поля "->dat" имеет смысл
*	только для переменной внутри правой части формулы подстановки:
*	в этом случае значение поля "->dat.ref" представляет собой указатель
*	на описание переменной из образца, значение которой подлежит копированию.
*
*	3) 'Пассивная' t-переменная. Внутри образца данный тип отвечает
*	вхождению переменной, левее которой уже имелись вхождения переменной
*	с тем же именем. Внутри правой части формулы подстановки данный тип
*	отвечает вхождению переменной, значение которого будет строиться переносом
*	(а не копированием) некоторого участка исходного выражения. Значение
*	поля "->dat.ref" представляет собой указатель на описание самого левого
*	вхождения переменной с данным именем в образец (для переменной из образца),
*	или указатель на описание переменной из образца, значение которой
*	подлежит переносу.
*	
*	4) 'Активная' e-переменная. Внутри образца данный тип отвечает
*	самому левому вхождению переменной с данным именем - при условии, что
*	значение этой переменной не обусловливается положением её левой границы.
*	Внутри правой части формулы подстановки данный тип отвечает
*	вхождению переменной, значение которого будет строиться копированием
*	(а не переносом) некоторого участка исходного выражения. Для переменной
*	из образца значение поля "->dat.ref" представляет собой указатель на
*	ближайшую слева от текущей литеры активную e-переменную из этого же образца
*	(либо NULL, если такой переменной нет). Для переменной из правой части
*	значение поля "->dat.ref" представляет собой указатель на описание
*	переменной из образца, значение которой подлежит копированию.
*	
*	5) 'Полуактивная' e-переменная. Данный тип используется только в образцах
*	и отвечает самому левому вхождению переменной с данным именем при условии,
*	что значение этой переменной однозначно определяется положением её левой
*	границы. Значение поля "->dat" игнорируется.
*	
*	6) 'Пассивная' e-переменная. Внутри образца данный тип отвечает
*	вхождению переменной, левее которой уже имелись вхождения переменной
*	с тем же именем. Внутри правой части формулы подстановки данный тип
*	отвечает вхождению переменной, значение которого будет строиться переносом
*	(а не копированием) некоторого участка исходного выражения. Значение
*	поля "->dat.ref" представляет собой указатель на описание самого левого
*	вхождения переменной с данным именем в образец (для переменной из образца),
*	или указатель на описание переменной из образца, значение которой
*	подлежит переносу.
*	
*	Значением поля "->ant" является указатель на литеру из поля зрения,
*	непосредственно предшествующую значению переменной. Значением поля "->sekv"
*	является указатель на литеру из поля зрения, непосредственно следующую
*	за значением переменной. Эти значения устанавливаются в ходе сопоставления.
*	
*	# ПРИМЕР: Рассмотрим формулу подстановки
*	#
*	#		s.1 (s.1) e.2 e.3 = s.1 e.2 e.2
*	#		1    2    3   4     5   6   7
*	#
*	# Переменные 1 и 3 являются 'активными' переменными образца;
*	# переменная 4 является 'полуактивной'; переменная 2 'пассивна' и отвечает
*	# 'активной' переменной 1. Переменные 5 и 6 являются 'пассивными'
*	# переменными из правой части - значение переменной 5 может быть получено
*	# переносом значения переменной 1 (или 2), а значение переменной 6
*	# получается переносом значения переменной 3. Напротив, переменная 7
*	# является 'активной' (ибо e.2 входит в образец лишь единожды) -
*	# и для построения её значения необходимо произвести копирование
*	# значения переменной 3.
*
*
*	III. При использовании литеры в качестве описания формулы подстановки,
*	значением поля "->ant" является указатель на первую литеру образца,
*	а значением поля "->sekv" является указатель на первую литеру правой части
*	(или NULL, если правая часть пуста).
*	
*	# Первой литерой образца всегда является левая функциональная скобка,
*	# а последней - правая функциональная скобка (между которыми заключается
*	# 'действительный' образец). При этом, вопреки общему правилу,
*	# значением поля "->dat" у открывающей образец левой функциональной скобки
*	# является указатель "->dat.ref" на парную правую функциональную скобку,
*	# завершающую образец.
*	#
*	# Последняя литера непустой правой части должна подчиняться условию
*	# "x->sekv == NULL".
*	
*	Значением поля "->dat.ref" является указатель на следующую формулу
*	подстановки (или NULL, если рассматриваемая формула является последней
*	в списке).
*
*/

typedef struct litero
{
	char tip;					/* Тип литеры */
	union
	{
		unsigned int cif;		/* Макроцифра */
		struct litero *ref;		/* Указатель на другую литеру */
		struct funkcio *funk;	/* Указатель на функцию */
	} dat;
	struct litero *ant;			/* Указатель на предыдущую литеру */
	struct litero *sekv;		/* Указатель на следующую литеру */
} LITERO;


# define CIFER 0			/* Макроцифра */
# define MD_LIT_PARENT 1		/* Левая литероидная скобка */
# define D_LIT_PARENT 2		/* Правая литероидная скобка */
# define MD_STR_PARENT 3		/* Левая структурная скобка */
# define D_STR_PARENT 4		/* Правая структурная скобка */
# define MD_FUNK_PARENT 5	/* Левая функциональная скобка */
# define D_FUNK_PARENT 6		/* Правая функциональная скобка */
# define VAR 7				/* Переменная */

# define ESTAS_CIFER(x) (x->tip == 0)
# define ESTAS_MD_LIT_PARENT(x) (x->tip == 1)
# define ESTAS_D_LIT_PARENT(x) (x->tip == 2)
# define ESTAS_MD_STR_PARENT(x) (x->tip == 3)
# define ESTAS_D_STR_PARENT(x) (x->tip == 4)
# define ESTAS_MD_FUNK_PARENT(x) (x->tip == 5)
# define ESTAS_D_FUNK_PARENT(x) (x->tip == 6)
# define ESTAS_VAR(x) (x->tip == 7)

# define S_AGEM 0			/* 'Активная' s-переменная */
# define S_MALAGEM 1		/* 'Пассивная' s-переменная */
# define T_AGEM 2			/* 'Активная' t-переменная */
# define T_MALAGEM 3		/* 'Пассивная' t-переменная */
# define E_AGEM 4			/* 'Активная' e-переменная */
# define E_DUONAGEM 5		/* 'Полуактивная' e-переменная */
# define E_MALAGEM 6		/* 'Пассивная' e-переменная */

# define ESTAS_S_AGEM(x) (x->tip == 0)
# define ESTAS_S_MALAGEM(x) (x->tip == 1)
# define ESTAS_T_AGEM(x) (x->tip == 2)
# define ESTAS_T_MALAGEM(x) (x->tip == 3)
# define ESTAS_E_AGEM(x) (x->tip == 4)
# define ESTAS_E_DUONAGEM(x) (x->tip == 5)
# define ESTAS_E_MALAGEM(x) (x->tip == 6)


/*
*	Структура FUNKCIO предназначена для записи функций рефал-машины.
*
*	Функция может быть встроенной или 'чисто рефальской'. Если функция
*	является встроенной, значение поля "->enk" равно 1; иначе оно равно 0.
*
*	Встроенные функции должны быть написаны на языке C в следующем формате:
*
*		void ИМЯ_ФУНКЦИИ (void);
*
*	Значением поля "->dat.f" является указатель на функцию указанного формата.
*
*	'Чисто рефальская' функция определяется непустым списком формул
*	подстановки. Значением поля "->dat.s" является указатель на первую
*	формулу подстановки из списка, отвечающего рассматриваемой функции.
*
*	Каждая функция (исключая единственную внутреннюю функцию, останавливающую
*	рефал-машину) имеет имя. Предполагается, что таковым является некоторое
*	непустое слово юникода; оно сохраняется в виде двунаправленного
*	списка литер с началом "->nom". Каждая из таких литер содержит в качестве
*	значения поля "->dat.cif" номер соответствующей буквы из имени функции.
*	При этом тип литер игнорируется. Значением поля "->sekv" для последней
*	литеры в этом списке является NULL. Значение поля "->ant" для первой
*	литеры в этом списке игнорируется.
*
*	Функции рефал-машины образуют список. Для его организации предназначено
*	поле "->sekv", значением которого является указатель на функцию,
*	следующую за рассматриваемой (либо NULL, если рассматриваемая функция
*	является последней в списке).
*	
*/

typedef struct funkcio
{
	char enk;				/* Тип функции */
	struct litero *nom;		/* Имя функции */
	union
	{
		void (*f) (void);	/* Указатель на встроенную функцию */
		struct litero *s;	/* Указатель на первую формулу подстановки */
	} dat;
	struct funkcio *sekv;	/* Указатель на следующую функцию */
} FUNKCIO;


extern int ARGC;		/* Первый аргумент основной функции */
extern char **ARGV;		/* Второй аргумент основной функции */


/*
*	Переменная "LLL" представляет собой указатель на начало списка
*	свободных литер. Данный список является однонаправленным;
*	значения полей "->ant" игнорируются.
*
*/

extern LITERO *LLL;


/*
*	Переменная "LF" представляет собой указатель на начало списка функций
*	рефал-машины.
*
*/

extern FUNKCIO *LF;


/*
*	Переменная "LKP" представляет собой указатель на начало списка
*	конкретизаций. Данный список является однонаправленным; значения
*	полей "->ant" игнорируются. Значениями полей "->dat.ref" являются
*	указатели на правые функциональные скобки внутри поля зрения.
*	Внутри списка эти указатели построены согласно порядку осуществления
*	соответствующих конкретизаций: "LKP->dat.ref" указывает на применяемую
*	в первую очередь, "(LKP->sekv)->dat.ref" - на следующую, и т.д.
*	Тип литер из данного списка игнорируется.
*
*/

extern LITERO *LKP;


/*
*	Переменная "KKK" представляет собой указатель на левую функциональную
*	скобку, отвечающую непосредственно осуществляемой конкретизации.
*
*/

extern LITERO *KKK;


/*
*	Переменная "KKP" представляет собой указатель на правую функциональную
*	скобку, отвечающую непосредственно осуществляемой конкретизации.
*
*/

extern LITERO *KKP;


/*
*	Переменная "KFS" представляет собой указатель на непосредственно
*	обрабатываемую формулу подстановки.
*
*/

extern LITERO *KFS;


/*
*	ПРИМЕЧАНИЕ: Тот факт, что макросы "ligi_literojn" и "ligi_parentezojn"
*	сначала изменяют параметры первого аргумента, и только потом -  второго,
*	является существенным и используется в ряде функций.
*
*/

# define ligi_literojn(x,y) {x->sekv = y; y->ant = x;}
# define ligi_parentezojn(x,y) {x->dat.ref = y; y->dat.ref = x;}


# define krei_literon(x) {\
	x = LLL;\
	if (LLL->sekv == NULL)\
	{\
		LLL = (LITERO *) malloc (sizeof (LITERO));\
		if (LLL == NULL)\
		{\
			exit (EXIT_FAILURE);\
		};\
		LLL->sekv = NULL;\
	}\
	else\
	{\
		LLL = LLL->sekv;\
	};\
}


/*
*	ПРИМЕЧАНИЕ: Факт сохранения старого значения поля "->ant" у 'стёртых' литер
*	является существенным и используется в ряде функций.
*
*/

# define detrui_literon(x) {x->sekv = LLL; LLL = x;}
